package mmql

import (
    "fmt"
    "io"
    "strings"
)

type LexerExpect struct {
    statements bool
    keywords bool
    brackets bool
    quotes bool
    key string
}

type Lexer struct {
    reader *strings.Reader
    readToken bool
}

func ( this *Lexer ) Parse( line string ) ( IStatement, error ) {

    this.reader = strings.NewReader( line )

    ex := LexerExpect{
        statements: true,
        keywords: false,
        brackets: true,
        quotes: true,
    }

    return this.ReadStatement( &ex )
}

func ( this *Lexer ) ReadStatement( expecting *LexerExpect ) ( IStatement, error ) {
    reader := this.reader

    var stmt IStatement

    for {
        r, _, err := reader.ReadRune()
        if err != nil {
            return nil, err
        }

        if expecting.quotes && strings.ContainsRune( K_QUOTES, r ) {
            s, err := this.ReadUntilClose( r, true, '\\' )
            if err != nil {
                return nil, err
            }

            stmt = QuotedValue{
                RawStatement: &RawStatement{ Value: s },
                Quote: string( r ),
            }
            return stmt, nil
        }

        bracketIndex := strings.IndexRune( K_BRACKETS, r )
        if expecting.brackets && bracketIndex % 2 == 0 {
            cl := rune( K_BRACKETS[ bracketIndex + 1 ] )
            s, err := this.ReadUntilClose( cl, false, 'A' )
            if err != nil {
                return nil, err
            }

            var b strings.Builder
            b.WriteRune( r )
            b.WriteRune( cl )

            stmt = BracketedValue{
                RawStatement: &RawStatement{ Value: s },
                Brackets: b.String(),
            }
            return stmt, nil
        }

        if expecting.statements {
        }
    }

    return nil, fmt.Errorf( "Nothing to read" )
}

func ( this *Lexer ) ReadUntilClose( closing rune, canEsc bool, escRune rune ) ( string, error ) {
    var b strings.Builder

    reader := this.reader
    for {
        r, _, err := reader.ReadRune()

        if err != nil {
            if err == io.EOF {
                return "", fmt.Errorf( "Unexpected end of data after opening %s", string( closing ) )
            }
            return "", err
        }

        if r == closing {
            break
        } else if canEsc && r == escRune {
            r2, _, err := reader.ReadRune()
            if err != nil {
                return "", err
            }

            if r2 != closing {
                b.WriteRune( r )
                b.WriteRune( r2 )
                continue
            }
        }

        b.WriteRune( r )
    }

    return b.String(), nil
}
